@@ -6,7 +6,7 @@ no  warnings 'uninitialized';
 
 use vars qw($VERSION);
 
-$VERSION = '1.45';
+$VERSION = '2.0';
 
 =head1 NAME
 
@@ -110,7 +110,7 @@ There are lots of potential uses for Yote, and a few come to mind :
 
  #
  # Anything that cannot trace a reference path to the root
- # is eligable for being removed upon compression. 
+ # is eligable for being removed upon compression.
  #
 
 =head1 PUBLIC METHODS
@@ -141,9 +141,10 @@ no warnings 'numeric';
 no warnings 'uninitialized';
 no warnings 'recursion';
 
-use Devel::Refcount 'refcount';
 use File::Copy;
-use WeakRef;
+use File::Path qw(make_path remove_tree);
+use Scalar::Util qw(weaken);
+
 use Module::Loaded;
 
 =head1 NAME
@@ -220,9 +221,15 @@ sub fetch {
     #
     # Return the object if we have a reference to its dirty state.
     #
-    my $ref = $self->{_DIRTY}{$id} || $self->{_WEAK_REFS}{$id};
+    my $ref = $self->{_DIRTY}{$id};
     if( defined $ref ) {
         return $ref;
+    } else {
+        $ref = $self->{_WEAK_REFS}{$id};
+        if( $ref ) {
+            return $ref;
+        }
+        undef $ref;
     }
     my $obj_arry = $self->{_DATASTORE}->_fetch( $id );
 
@@ -231,14 +238,12 @@ sub fetch {
         if( $class eq 'ARRAY' ) {
             my( @arry );
             tie @arry, 'Yote::Array', $self, $id, @$data;
-            my $tied = tied @arry; $tied->[3] = \@arry;
             $self->_store_weak( $id, \@arry );
             return \@arry;
         }
         elsif( $class eq 'HASH' ) {
             my( %hash );
             tie %hash, 'Yote::Hash', $self, $id, map { $_ => $data->{$_} } keys %$data;
-            my $tied = tied %hash; $tied->[3] = \%hash;
             $self->_store_weak( $id, \%hash );
             return \%hash;
         }
@@ -262,104 +267,104 @@ sub fetch {
     return undef;
 } #fetch
 
-=head2 run_recycler
+=head2 run_purger
 
 =cut
-sub run_recycler {
-    my $self = shift;
-     $self->stow_all();
-    $self->{_DATASTORE}->_recycle_objects();
-} #run_recycler
+sub run_purger {
+    my( $self, $make_tally, $copy_only ) = @_;
+    $self->stow_all();
 
-=head2 compress_store
+    my $keep_db = $self->{_DATASTORE}->_generate_keep_db();
 
-This creates a new data store with all objects
-removed that cannot be traced back to the root.
+    # analyze to see what percentage would be kept
+    my $total = $keep_db->entry_count;
+    my $keep = 0;
+    for my $tid (1..$total) {
+        my( $has_keep ) = $keep_db->get_record( $tid )->[0];
+        $keep++ if $has_keep;
+    }
 
-This will fail if the following directories 
-exist :
-  ${ROOT-DIRECOTRY-FOR-OBJECT-STORE}_COMPRESS_BAK
-  ${ROOT-DIRECOTRY-FOR-OBJECT-STORE}_NEW_RECYC
+    #
+    # If there are more things to keep than not, do a db purge,
+    # otherwise, rebuild the db.
+    #
+    my $do_purge = $keep > ( $total/2 ) && ! $copy_only;
+    my $purged;
+    if( $do_purge ) {
+        $purged = $self->{_DATASTORE}->_purge_objects( $keep_db, $make_tally );
+    } else {
+        $purged = $self->_copy_active_ids( $keep_db, $make_tally );
+    }
 
-The first directory is where the original object
-store gets backed up after the compression. If 
-the compression is a success, it should be removed.
+    $self->{_DATASTORE}->_update_recycle_ids( $keep_db );
 
-The second is a temporary working directory for the
-new copied over database.
+    # commenting out for a test
+    $keep_db->unlink_store;
 
-This requires enough room on the file system
-to potentially double the size of the database
-temporarily as it is copied to a new one.
+    $purged;
+} #run_purger
 
-=cut
-sub compress_store {
-    my( $self, $newdir, $backdir ) = @_;
+sub _copy_active_ids {
+    my( $self, $copy_db ) = @_;
     $self->stow_all();
 
     my $original_dir = $self->{args}{store};
-    $backdir //= $original_dir . '_COMPRESS_BAK';
-    $newdir  //= $original_dir . '_NEW_RECYC';
+    my $backdir = $original_dir . '_COMPRESS_BACK_RECENT';
+    my $newdir  = $original_dir . '_NEW_RECYC';
 
-    if( -x $backdir ) {
-        die "Unable to run compress store, backup directory '$backdir' already exists.";
+    if( -e $backdir ) {
+        my $oldback = $original_dir . '_COMPRESS_BACK_OLD';
+        if( -d $oldback ) {
+            warn "Removing old compression backup directory";
+            remove_tree( $oldback );
+        }
+        move( $backdir, $oldback ) or die $!;
     }
 
     if( -x $newdir ) {
         die "Unable to run compress store, temp directory '$newdir' already exists.";
     }
-
-    if( $self->_has_weak_refs ) {
-        die "Unable to run compress store. There are still outstanding references to yote objects that would be deleted during the compress.";
-    }
     my $newstore = Yote::ObjStore->_new( { store => $newdir } );
 
-    my( @copy_ids ) = ( $self->_first_id );
+    my( @purges );
+    for my $keep_id ( 1..$copy_db->entry_count ) {
 
-    my $count = 0;
-    my( %seen );
-    while( @copy_ids ) {
-        my $id = shift @copy_ids;
-#        next if $seen{$id}++;
-        next if $newstore->{_DATASTORE}{DATA_STORE}->has_id( $id ) && $id != $self->_first_id;
-        next if$id == $self->_first_id && $count > 0;
-        
-        print STDERR "\t$id";
-        if( ++$count > 80 ) {
-            print STDERR "\n";
-            $count = 0;
+        my( $has_keep ) = $copy_db->get_record( $keep_id )->[0];
+        if( $has_keep ) {
+            my $obj = $self->fetch( $keep_id );
+
+            $newstore->{_DATASTORE}{DATA_STORE}->ensure_entry_count( $keep_id - 1 );
+            $newstore->_dirty( $obj, $keep_id );
+            $newstore->_stow( $obj, $keep_id );
+        } elsif( $self->{_DATASTORE}{DATA_STORE}->has_id( $keep_id ) ) {
+            push @purges, $keep_id;
         }
-        
-        my $obj = $self->fetch( $id );
-#        $obj->{STORE} = $newstore;
+    } #each entry id
 
-        $newstore->{_DATASTORE}{DATA_STORE}->ensure_entry_count( $id - 1 );
-        $newstore->_dirty( $obj, $id );
-        $newstore->_stow( $obj, $id );
-
-        my $r = ref( $obj );
-        if ( $r eq 'ARRAY' ) {
-            my $tied = tied @$obj;
-            push @copy_ids, grep { $_ > 0 } @{$tied->[1]};
-        } elsif ( $r eq 'HASH' ) {
-            my $tied = tied %$obj;
-            push @copy_ids, grep { $_ > 0 } values %{$tied->[1]};
-        } else {
-            push @copy_ids, grep { $_ > 0 } values %{$obj->{DATA}};
-        }
-    }
-    
     move( $original_dir, $backdir ) or die $!;
     move( $newdir, $original_dir ) or die $!;
 
-} #compress_store
+    \@purges;
+
+} #_copy_active_ids
+
+=head2 has_id
+
+ Returns true if there is a valid reference linked to the id
+
+=cut
+sub has_id {
+    my( $self, $id ) = @_;
+    return $self->{_DATASTORE}{DATA_STORE}->has_id( $id );
+}
+
 =head2 stow_all
 
  Saves all newly created or dirty objects.
 
 =cut
 sub stow_all {
-    my $self = $_[0];
+    my $self = shift;
     my @odata;
     for my $obj (values %{$self->{_DIRTY}} ) {
         my $cls;
@@ -371,13 +376,38 @@ sub stow_all {
         } else {
             $cls = $ref;
         }
-        push( @odata, [ $self->_get_id( $obj ), $cls, $self->_raw_data( $obj ) ] );
+        my( $text_rep ) = $self->_raw_data( $obj );
+        push( @odata, [ $self->_get_id( $obj ), $cls, $text_rep ] );
     }
     $self->{_DATASTORE}->_stow_all( \@odata );
     $self->{_DIRTY} = {};
 } #stow_all
 
 
+=head2 stow( $obj )
+
+ Saves that object to the database
+
+=cut
+sub stow {
+    my( $self, $obj ) = @_;
+    my $cls;
+    my $ref = ref( $obj );
+    if( $ref eq 'ARRAY' || $ref eq 'Yote::Array' ) {
+        $cls = 'ARRAY';
+    } elsif( $ref eq 'HASH' || $ref eq 'Yote::Hash' ) {
+        $cls = 'HASH';
+    } else {
+        $cls = $ref;
+    }
+    my $id = $self->_get_id( $obj );
+    my( $text_rep ) = $self->_raw_data( $obj );
+    $self->{_DATASTORE}->_stow( $id, $cls, $text_rep );
+    delete $self->{_DIRTY}{$id};
+} #stow
+
+
+
 # -------------------------------
 #      * PRIVATE METHODS *
 # -------------------------------
@@ -403,6 +433,11 @@ sub _init {
 }
 
 
+sub dirty_count {
+    my $self = shift;
+    return scalar( keys %{$self->{_DIRTY}} );
+}
+
 #
 # Markes given object as dirty.
 #
@@ -419,6 +454,7 @@ sub _first_id {
 } #_first_id
 
 sub _get_id {
+    # for debugging I think?
     shift->__get_id( shift );
 }
 
@@ -427,7 +463,7 @@ sub __get_id {
 
     my $class = ref( $ref );
     die "__get_id requires reference. got '$ref'" unless $class;
-    
+
     if( $class eq 'Yote::Array') {
         return $ref->[0];
     }
@@ -440,7 +476,6 @@ sub __get_id {
         my( @data ) = @$ref;
         my $id = $self->{_DATASTORE}->_get_id( $class );
         tie @$ref, 'Yote::Array', $self, $id;
-        $tied = tied @$ref; $tied->[3] = $ref;
         push( @$ref, @data );
         $self->_dirty( $ref, $id );
         $self->_store_weak( $id, $ref );
@@ -459,8 +494,8 @@ sub __get_id {
         my $id = $self->{_DATASTORE}->_get_id( $class );
 
         my( %vals ) = %$ref;
+
         tie %$ref, 'Yote::Hash', $self, $id;
-        $tied = tied %$ref; $tied->[3] = $ref;
         for my $key (keys %vals) {
             $ref->{$key} = $vals{$key};
         }
@@ -489,19 +524,19 @@ sub _stow {
     $id //= $self->_get_id( $obj );
     die unless $id;
 
-    my $data = $self->_raw_data( $obj );
+    my( $text_rep, $data ) = $self->_raw_data( $obj );
 
     if( $class eq 'ARRAY' ) {
-        $self->{_DATASTORE}->_stow( $id,'ARRAY', $data );
+        $self->{_DATASTORE}->_stow( $id,'ARRAY', $text_rep );
         $self->_clean( $id );
     }
     elsif( $class eq 'HASH' ) {
-        $self->{_DATASTORE}->_stow( $id,'HASH',$data );
+        $self->{_DATASTORE}->_stow( $id,'HASH',$text_rep );
         $self->_clean( $id );
     }
     elsif( $class eq 'Yote::Array' ) {
         if( $self->_is_dirty( $id ) ) {
-            $self->{_DATASTORE}->_stow( $id,'ARRAY',$data );
+            $self->{_DATASTORE}->_stow( $id,'ARRAY',$text_rep );
             $self->_clean( $id );
         }
         for my $child (@$data) {
@@ -512,7 +547,7 @@ sub _stow {
     }
     elsif( $class eq 'Yote::Hash' ) {
         if( $self->_is_dirty( $id ) ) {
-            $self->{_DATASTORE}->_stow( $id, 'HASH', $data );
+            $self->{_DATASTORE}->_stow( $id, 'HASH', $text_rep );
         }
         $self->_clean( $id );
         for my $child (values %$data) {
@@ -523,7 +558,7 @@ sub _stow {
     }
     else {
         if( $self->_is_dirty( $id ) ) {
-            $self->{_DATASTORE}->_stow( $id, $class, $data );
+            $self->{_DATASTORE}->_stow( $id, $class, $text_rep );
             $self->_clean( $id );
         }
         for my $val (values %$data) {
@@ -564,20 +599,6 @@ sub _is_dirty {
     $ans;
 } #_is_dirty
 
-sub _has_weak_refs {
-    my $self = shift;
-
-    # if there is an ARRAY or HASH, since it is tied, it has a reference to itself
-    0 < grep { $_ && refcount($_) > (ref($_) =~ /^(ARRAY|HASH)$/ ? 1 : 0 ) }
-        values %{$self->{_WEAK_REFS}};
-}
-
-sub _purge {
-    my $self = shift;
-    $self->{_DIRTY} = {};
-    $self->{_WEAK_REFS} = {};
-}
-
 #
 # Returns data structure representing object. References are integers. Values start with 'v'.
 #
@@ -587,10 +608,12 @@ sub _raw_data {
     return unless $class;
     my $id = $self->_get_id( $obj );
     die unless $id;
+    my( $r, $is_array );
     if( $class eq 'ARRAY' ) {
         my $tied = tied @$obj;
         if( $tied ) {
-            return $tied->[1];
+            $r = $tied->[1];
+            $is_array = 1;
         } else {
             die;
         }
@@ -598,23 +621,30 @@ sub _raw_data {
     elsif( $class eq 'HASH' ) {
         my $tied = tied %$obj;
         if( $tied ) {
-            return $tied->[1];
+            $r = $tied->[1];
         } else {
             die;
         }
     }
     elsif( $class eq 'Yote::Array' ) {
-        return $obj->[1];
+        $r = $obj->[1];
+        $is_array = 1;
     }
     elsif( $class eq 'Yote::Hash' ) {
-        return $obj->[1];
+        $r = $obj->[1];
     }
     else {
-        return $obj->{DATA};
+        $r = $obj->{DATA};
     }
 
+    if( $is_array ) {
+        return join( "`", map { if( defined($_) ) { s/[\\]/\\\\/gs; s/`/\\`/gs; } $_ } @$r ), $r;
+    }
+    return join( "`", map { if( defined($_) ) { s/[\\]/\\\\/gs; s/`/\\`/gs; } $_ } %$r ), $r;
+
 } #_raw_data
 
+
 sub _store_weak {
     my( $self, $id, $ref ) = @_;
     die unless $ref;
@@ -701,7 +731,10 @@ sub set {
         $self->{STORE}->_dirty( $self, $self->{ID} );
     }
 
-
+    unless( defined $inval ) {
+        delete $self->{DATA}{$fld};
+        return;
+    }
     $self->{DATA}{$fld} = $inval;
     return $self->{STORE}->_xform_out( $self->{DATA}{$fld} );
 } #set
@@ -816,8 +849,11 @@ sub AUTOLOAD {
             my( $self, $val ) = @_;
             my $inval = $self->{STORE}->_xform_in( $val );
             $self->{STORE}->_dirty( $self, $self->{ID} ) if $self->{DATA}{$fld} ne $inval;
+            unless( defined $inval ) {
+                delete $self->{DATA}{$fld};
+                return;
+            }
             $self->{DATA}{$fld} = $inval;
-
             return $self->{STORE}->_xform_out( $self->{DATA}{$fld} );
         };
         goto &$AUTOLOAD;
@@ -909,78 +945,11 @@ sub _instantiate {
     bless { ID => $_[1], DATA => {}, STORE => $_[2] }, $_[0];
 } #_instantiate
 
-sub DESTROY {}
-
-sub _DUMP_ALL {
-    my( $self, $seen, $show ) = @_;
-    $seen //= {};
-    $show //= {};
-
-    delete $show->{$self->{ID}};
-    $seen->{$self->{ID}} = 1;
-    my $buf = $self->_DUMP;
-
-    for my $obj_id (sort { $a <=> $b } grep { index($_,'v') != 0 && 0 == $seen->{$_}++ } values %{$self->{DATA}}) {
-        $show->{$obj_id} = 1;
-    }
-
-    while(1) {
-        my( $obj_id ) = keys %$show;
-        last unless $obj_id;
-        $buf .= "-------------------------\n";
-
-        my $obj = $self->{STORE}->fetch( $obj_id );
-        my $r = ref( $obj );
-
-        if( $r eq 'ARRAY' ) {
-            $buf .= "$obj_id (ARRAY)\n";
-            my $tied = tied @$obj;
-            delete $show->{$obj_id};
-            for my $item (@{$tied->[1]}) {
-                if( $item > 0 ) {
-                    $show->{$item} = 1;
-                    $buf .= "\t* $item\n";
-                } else {
-                    $buf .= "\t".substr($item,1)."\n";
-                }
-            }
-        }
-        elsif( $r eq 'HASH' ) {
-            $buf .= "$obj_id (HASH)\n";
-            delete $show->{$obj_id};
-            my $tied = tied %$obj;
-            my $th = $tied->[1];
-            for my $key (keys %$th) {
-                my $item = $th->{$key};
-
-                if( $item > 0 ) {
-                    $show->{$item} = 1;
-                    $buf .= "\t$key -> * $item\n";
-                } else {
-                    $buf .= "\t$key -> ".substr($item,1)."\n";
-                }
-            }
-        }
-        else {
-            $buf .= $obj->_DUMP_ALL( $seen, $show );
-        }
-    }
-    $buf;
-} #_DUMP_ALL
-
-sub _DUMP {
+sub DESTROY {
     my $self = shift;
-    my $buf = "$self->{ID} (".ref($self).")\n";
-    for my $key (sort keys %{$self->{DATA}}) {
-        my $val = $self->{DATA}{$key};
-        if( index( $val, 'v' ) != 0 ) {
-            $buf .= "\t$key -> * $val \n";
-        } else {
-            $buf .= "\t$key -> ".substr($val,1)."\n";
-        }
-    }
-    $buf;
-} #_DUMP
+    delete $self->{STORE}{_WEAK_REFS}{$self->{ID}};
+}
+
 
 # ---------------------------------------------------------------------------------------------------------------------
 
@@ -1022,7 +991,7 @@ sub FETCHSIZE {
 
 sub STORE {
     my( $self, $idx, $val ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     $self->[1][$idx] = $self->[2]->_xform_in( $val );
 }
 sub STORESIZE {}  #stub for array
@@ -1033,53 +1002,56 @@ sub EXISTS {
 }
 sub DELETE {
     my( $self, $idx ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     delete $self->[1][$idx];
 }
 
 sub CLEAR {
     my $self = shift;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     @{$self->[1]} = ();
 }
 sub PUSH {
     my( $self, @vals ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     push( @{$self->[1]}, map { $self->[2]->_xform_in($_) } @vals );
 }
 sub POP {
     my $self = shift;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     return $self->[2]->_xform_out( pop @{$self->[1]} );
 }
 sub SHIFT {
     my( $self ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     my $val = splice @{$self->[1]}, 0, 1;
     return $self->[2]->_xform_out( $val );
 }
 sub UNSHIFT {
     my( $self, @vals ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     unshift @{$self->[1]}, map {$self->[2]->_xform_in($_)} @vals;
 }
 sub SPLICE {
     my( $self, $offset, $length, @vals ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     return map { $self->[2]->_xform_out($_) } splice @{$self->[1]}, $offset, $length, map {$self->[2]->_xform_in($_)} @vals;
 }
 sub EXTEND {}
 
-sub DESTROY {}
+sub DESTROY {
+    my $self = shift;
+    delete $self->[2]->{_WEAK_REFS}{$self->[0]};
+}
 
-# ---------------------------------------------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------------
 
 package Yote::Hash;
 
-############################################################################################################
-# This module is used transparently by Yote to link hashes into its graph structure. This is not meant to  #
-# be called explicitly or modified.									   #
-############################################################################################################
+######################################################################################
+# This module is used transparently by Yote to link hashes into its graph structure. #
+# This is not meant to  be called explicitly or modified.                            #
+######################################################################################
 
 use strict;
 use warnings;
@@ -1102,7 +1074,7 @@ sub TIEHASH {
 
 sub STORE {
     my( $self, $key, $val ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     $self->[1]{$key} = $self->[2]->_xform_in( $val );
 }
 
@@ -1129,18 +1101,21 @@ sub EXISTS {
 }
 sub DELETE {
     my( $self, $key ) = @_;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     return delete $self->[1]{$key};
 }
 sub CLEAR {
     my $self = shift;
-    $self->[2]->_dirty( $self->[3], $self->[0] );
+    $self->[2]->_dirty( $self->[2]{_WEAK_REFS}{$self->[0]}, $self->[0] );
     %{$self->[1]} = ();
 }
 
-sub DESTROY {}
+sub DESTROY {
+    my $self = shift;
+    delete $self->[2]->{_WEAK_REFS}{$self->[0]};
+}
 
-# ---------------------------------------------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------------
 
 package Yote::YoteDB;
 
@@ -1151,11 +1126,7 @@ no warnings 'uninitialized';
 
 use Data::RecordStore;
 
-use WeakRef;
 use File::Path qw(make_path);
-use JSON;
-
-use Devel::Refcount 'refcount';
 
 use constant {
   DATA => 2,
@@ -1169,10 +1140,21 @@ sub open {
   my( $pkg, $obj_store, $args ) = @_;
   my $class = ref( $pkg ) || $pkg;
 
+  my $DATA_STORE;
+  eval {
+      $DATA_STORE = Data::RecordStore->open( $args->{ store } );
+      
+  };
+  if( $@ ) {
+      if( $@ =~ /old format/ ) {
+          die "This yote store is of an older format. It can be converted using the yote_explorer";
+      }
+      die $@;
+  }
   my $self = bless {
       args       => $args,
       OBJ_STORE  => $obj_store,
-      DATA_STORE => Data::RecordStore->open( $args->{ store } ),
+      DATA_STORE => $DATA_STORE,
   }, $class;
   $self->{DATA_STORE}->ensure_entry_count( 1 );
   $self;
@@ -1186,14 +1168,49 @@ sub open {
 sub _fetch {
   my( $self, $id ) = @_;
   my $data = $self->{DATA_STORE}->fetch( $id );
+
   return undef unless $data;
 
   my $pos = index( $data, ' ' ); #there is a always a space after the class.
+  $pos = ( length( $data ) ) if $pos == -1;
   die "Malformed record '$data'" if $pos == -1;
   my $class = substr $data, 0, $pos;
   my $val   = substr $data, $pos + 1;
   my $ret = [$id,$class,$val];
-  $ret->[DATA] = from_json( $ret->[DATA] );
+
+  # so foo` or foo\\` but not foo\\\`
+  # also this will never start with a `
+  my $parts = [ split /\`/, $val, -1 ];
+
+  # check to see if any of the parts were split on escapes
+  if( 0 < grep { /\\$/ } @$parts ) {
+      my $newparts = [];
+      my $shim = '';
+      for my $part (@$parts) {
+          if( $part =~ /(^|[^\\]((\\\\)+)?)$/ ) {
+              my $newpart = $shim ? "$shim\`$part" : $part;
+              $newpart =~ s/\\`/`/gs;
+              $newpart =~ s/\\\\/\\/gs;
+              push @$newparts, $newpart;
+              $shim = '';
+          } else {
+              $shim = $shim ? "$shim\`$part" : $part;
+          }
+      }
+      if( $shim ) {
+          $shim =~ s/\\`/`/gs;
+          $shim =~ s/\\\\/\\/gs;
+          push @$newparts, $shim;
+      }
+      $parts = $newparts;
+  }
+
+  if( $class eq 'ARRAY' ) {
+      $ret->[DATA] = $parts;
+  } else {
+      $ret->[DATA] = { @$parts };
+  }
+
   $ret;
 } #_fetch
 
@@ -1219,88 +1236,143 @@ sub _max_id {
   shift->{DATA_STORE}->entry_count;
 }
 
-sub _recycle_objects {
-  my $self = shift;
+sub _generate_keep_db {
+    my $self = shift;
+    my $mark_to_keep_store = Data::RecordStore::FixedStore->open( "I", $self->{args}{store} . '/PURGE_KEEP' );
 
-  my $mark_to_keep_store = Data::RecordStore::FixedStore->open( "I", $self->{args}{store} . '/RECYCLE' );
-  $mark_to_keep_store->empty();
-  $mark_to_keep_store->ensure_entry_count( $self->{DATA_STORE}->entry_count );
-  
-  # the already deleted cannot be re-recycled
-  my $ri = $self->{DATA_STORE}->_get_recycled_ids;
-  for ( @$ri ) {
-    $mark_to_keep_store->put_record( $_, [ 1 ] );
-  }
+    $mark_to_keep_store->empty();
+    $mark_to_keep_store->ensure_entry_count( $self->{DATA_STORE}->entry_count );
 
-  my $trace_to_root = sub {
-      my( $keep_id, $mark_to_keep_store ) = @_;
-      my( @queue ) = ( $keep_id );
+    my $check_store = Data::RecordStore::FixedStore->open( "L", $self->{args}{store} . '/CHECK' );
+    $check_store->empty();
 
-      $mark_to_keep_store->put_record( $keep_id, [ 1 ] );
+    $mark_to_keep_store->put_record( 1, [ 1 ] );
 
-      # get the object ids referenced by this keeper object
-      while( @queue ) {
-          $keep_id = shift @queue;
+    my( %seen );
+    my( @checks ) = ( 1 );
 
-          my $item = $self->_fetch( $keep_id );
-          my( @additions );
-          if ( ref( $item->[DATA] ) eq 'ARRAY' ) {
-              ( @additions ) = grep { /^[^v]/ } @{$item->[DATA]};
-          } else {
-              ( @additions ) = grep { /^[^v]/ } values %{$item->[DATA]};
-          }
+    for my $referenced_id ( grep { defined($self->{OBJ_STORE}{_WEAK_REFS}{$_}) } keys %{ $self->{OBJ_STORE}{_WEAK_REFS} } ) {
+        push @checks, $referenced_id;
+    }
 
-          for my $keeper ( @additions ) {
-              next if $mark_to_keep_store->get_record( $keeper )->[0];
-              $mark_to_keep_store->put_record( $keeper, [ 1 ] );
-              push @queue, $keeper;
-          }
-      } #while there is a queue
-  };
 
-  &$trace_to_root( $self->_first_id, $mark_to_keep_store );
+    #
+    # While there are items to check, check them.
+    #
+    while( @checks || $check_store->entry_count > 0 ) {
+        my $check_id = shift( @checks ) || $check_store->pop->[0];
+        $mark_to_keep_store->put_record( $check_id, [ 1 ] );
 
-  #
-  # If there are any entries in the weak references, do not recycle these.
-  # This ignores the possibility of circular references, but that uncommon case
-  # is not worth the complexity.
-  #
-  for my $referenced_id ( grep { defined($self->{OBJ_STORE}{_WEAK_REFS}{$_}) } keys %{ $self->{OBJ_STORE}{_WEAK_REFS} } ) {
-      # make sure that these are actually referenced. They may be 
-      # in DIRTY, and, if they are Yote::Array or Yote::Hash, they
-      # have an extra reference due to the tie.
-      my $obj = $self->{OBJ_STORE}->fetch( $referenced_id );
-      my $min_ref_count = 1;
-      if( $self->{OBJ_STORE}->_is_dirty( $referenced_id ) ) {
-          $min_ref_count++;
-      }
-      $min_ref_count++ if ref($obj) =~ /^(ARRAY|HASH)$/;
+        my $item = $self->_fetch( $check_id );
+        $seen{$check_id} = 1;
+        my( @additions );
+        if ( ref( $item->[DATA] ) eq 'ARRAY' ) {
+            ( @additions ) = grep { /^[^v]/ && ! $seen{$_}++ } @{$item->[DATA]};
+        } else {
+            ( @additions ) = grep { /^[^v]/ && ! $seen{$_}++ } values %{$item->[DATA]};
+        }
+        if( @checks > 1_000_000 ) {
+            for my $cid (@checks) {
+                my( $has_keep ) = $mark_to_keep_store->get_record( $cid )->[0];
+                unless( $has_keep ) {
+                    $check_store->push( [ $cid ] );
+                }
+            }
+            splice @checks;
+        }
+        if( scalar( keys(%seen) ) > 1_000_000 ) {
+            %seen = ();
+        }
+        push @checks, @additions;
+    }
+    $check_store->unlink_store;
 
-      if( refcount($obj) > $min_ref_count ) {
-          &$trace_to_root( $referenced_id, $mark_to_keep_store );
-      }
-  }
+    $mark_to_keep_store;
 
-  # the purge begins here
-  my $cands = $self->{DATA_STORE}->entry_count;
-  my $count = 0;
-  for my $cand ( 1..$cands) { #iterate each id in the entire object store
+} #_generate_keep_db
+
+#
+# Checks to see if the last entries of the stores can be popped off, making the purging quicker
+#
+sub _truncate_dbs {
+    my( $self, $mark_to_keep_store, $keep_tally ) = @_;
+    #loop through each database
+    my $stores = $self->{DATA_STORE}->all_stores;
+    my( @purged );
+    for my $store (@$stores) {
+        my $fn = $store->{FILENAME}; $fn =~ s!/[^/]+$!!;
+        my $keep;
+        while( ! $keep && $store->entry_count ) {
+            my( $check_id ) = @{ $store->get_record($store->entry_count) };
+            ( $keep ) = $mark_to_keep_store->get_record( $check_id )->[0];
+            if( ! $keep ) {
+                if( $self->{DATA_STORE}->delete( $check_id ) ) {
+                    if( $keep_tally ) {
+                        push @purged, $check_id;
+                    }
+                    $mark_to_keep_store->put_record( $check_id, [ 2 ] ); #mark as already removed by truncate
+                }
+            }
+        }
+    }
+    \@purged;
+}
+
+
+sub _update_recycle_ids {
+    my( $self, $mark_to_keep_store ) = @_;
+
+    return unless $mark_to_keep_store->entry_count > 0;
+
+    my $store = $self->{DATA_STORE};
+
+
+    # find the higest still existing ID and cap the index to this
+    my $highest_keep_id;
+    for my $cand (reverse ( 1..$mark_to_keep_store->entry_count )) {
+        my( $keep ) = $mark_to_keep_store->get_record( $cand )->[0];
+        if( $keep ) {
+            $store->set_entry_count( $cand );
+            $highest_keep_id = $cand;
+            last;
+        }
+    }
+
+    $store->empty_recycler;
+
+    # iterate each id in the entire object store and add those
+    # not marked for keeping into the recycling
+    for my $cand (reverse( 1.. $highest_keep_id) ) {
+        my( $keep ) = $mark_to_keep_store->get_record( $cand )->[0];
+        unless( $keep ) {
+            $store->recycle( $cand );
+        }
+    }
+} #_update_recycle_ids
+
+
+sub _purge_objects {
+  my( $self, $mark_to_keep_store, $keep_tally ) = @_;
+
+  my $purged = $self->_truncate_dbs( $mark_to_keep_store );
+
+  for my $cand ( 1..$mark_to_keep_store->entry_count) { #iterate each id in the entire object store
     my( $keep ) = $mark_to_keep_store->get_record( $cand )->[0];
 
-    die "Tried to recycle root entry" if $cand == 1 && ! $keep;
+    die "Tried to purge root entry" if $cand == 1 && ! $keep;
     if ( ! $keep ) {
-        $self->{DATA_STORE}->recycle( $cand );
-        delete $self->{OBJ_STORE}{_WEAK_REFS}{$cand};
-        $count++;
+        if( $self->{DATA_STORE}->delete( $cand ) ) {
+            $mark_to_keep_store->put_record( $cand, [ 3 ] ); #mark as already removed by purge
+            if( $keep_tally ) {
+                push @$purged, $cand;
+            }
+        }
     }
   }
 
-  # remove temporary recycle datastore
-  $mark_to_keep_store->unlink_store;
+  $purged;
 
-  $self->{DATA_STORE}->_get_recycled_ids;
-  
-} #_recycle_objects
+} #_purge_objects
 
 
 #
@@ -1308,7 +1380,7 @@ sub _recycle_objects {
 #
 sub _stow { #Yote::YoteDB::_stow
   my( $self, $id, $class, $data ) = @_;
-  my $save_data = "$class " . to_json($data);
+  my $save_data = "$class $data";
   $self->{DATA_STORE}->stow( $save_data, $id );
 } #_stow
 
@@ -1338,6 +1410,6 @@ __END__
        under the same terms as Perl itself.
 
 =head1 VERSION
-       Version 1.44  (Nov 23, 2016))
+       Version 2.0  (Nov 23, 2016))
 
 =cut
